/// <reference types="node" />
import { CommandClasses, type IZWaveEndpoint, type MaybeNotKnown, type MessageOrCCLogEntry } from "@zwave-js/core/safe";
import type { ZWaveApplicationHost, ZWaveHost } from "@zwave-js/host/safe";
import { PhysicalCCAPI } from "../lib/API";
import { type CCCommandOptions, CommandClass, type CommandClassDeserializationOptions } from "../lib/CommandClass";
import { AssociationGroupInfoCommand, AssociationGroupInfoProfile } from "../lib/_Types";
export declare const AssociationGroupInfoCCValues: Readonly<{
    commands: ((groupId: number) => {
        readonly meta: {
            readonly type: "any";
            readonly readable: true;
            readonly writeable: true;
        };
        readonly id: {
            commandClass: (typeof CommandClasses)["Association Group Information"];
            property: "issuedCommands";
            propertyKey: number;
        };
        readonly endpoint: (endpoint?: number | undefined) => {
            readonly commandClass: (typeof CommandClasses)["Association Group Information"];
            readonly endpoint: number;
            readonly property: "issuedCommands";
            readonly propertyKey: number;
        };
    }) & {
        is: (valueId: import("@zwave-js/core/safe").ValueID) => boolean;
        readonly options: {
            readonly stateful: true;
            readonly secret: false;
            readonly minVersion: 1;
            readonly supportsEndpoints: true;
            readonly autoCreate: true;
            readonly internal: true;
        };
    };
    groupInfo: ((groupId: number) => {
        readonly meta: {
            readonly type: "any";
            readonly readable: true;
            readonly writeable: true;
        };
        readonly id: {
            commandClass: (typeof CommandClasses)["Association Group Information"];
            property: "info";
            propertyKey: number;
        };
        readonly endpoint: (endpoint?: number | undefined) => {
            readonly commandClass: (typeof CommandClasses)["Association Group Information"];
            readonly endpoint: number;
            readonly property: "info";
            readonly propertyKey: number;
        };
    }) & {
        is: (valueId: import("@zwave-js/core/safe").ValueID) => boolean;
        readonly options: {
            readonly stateful: true;
            readonly secret: false;
            readonly minVersion: 1;
            readonly supportsEndpoints: true;
            readonly autoCreate: true;
            readonly internal: true;
        };
    };
    groupName: ((groupId: number) => {
        readonly meta: {
            readonly type: "any";
            readonly readable: true;
            readonly writeable: true;
        };
        readonly id: {
            commandClass: (typeof CommandClasses)["Association Group Information"];
            property: "name";
            propertyKey: number;
        };
        readonly endpoint: (endpoint?: number | undefined) => {
            readonly commandClass: (typeof CommandClasses)["Association Group Information"];
            readonly endpoint: number;
            readonly property: "name";
            readonly propertyKey: number;
        };
    }) & {
        is: (valueId: import("@zwave-js/core/safe").ValueID) => boolean;
        readonly options: {
            readonly stateful: true;
            readonly secret: false;
            readonly minVersion: 1;
            readonly supportsEndpoints: true;
            readonly autoCreate: true;
            readonly internal: true;
        };
    };
    hasDynamicInfo: {
        readonly id: {
            commandClass: (typeof CommandClasses)["Association Group Information"];
            property: "hasDynamicInfo";
        };
        readonly endpoint: (endpoint?: number | undefined) => {
            readonly commandClass: (typeof CommandClasses)["Association Group Information"];
            readonly endpoint: number;
            readonly property: "hasDynamicInfo";
        };
        readonly is: (valueId: import("@zwave-js/core/safe").ValueID) => boolean;
        readonly meta: {
            readonly type: "any";
            readonly readable: true;
            readonly writeable: true;
        };
        readonly options: {
            readonly stateful: true;
            readonly secret: false;
            readonly minVersion: 1;
            readonly supportsEndpoints: true;
            readonly autoCreate: true;
            readonly internal: true;
        };
    };
}>;
export declare class AssociationGroupInfoCCAPI extends PhysicalCCAPI {
    supportsCommand(cmd: AssociationGroupInfoCommand): MaybeNotKnown<boolean>;
    getGroupName(groupId: number): Promise<MaybeNotKnown<string>>;
    reportGroupName(groupId: number, name: string): Promise<void>;
    getGroupInfo(groupId: number, refreshCache?: boolean): Promise<{
        mode: number;
        profile: number;
        eventCode: number;
        hasDynamicInfo: boolean;
    } | undefined>;
    reportGroupInfo(options: AssociationGroupInfoCCInfoReportSpecificOptions): Promise<void>;
    getCommands(groupId: number, allowCache?: boolean): Promise<MaybeNotKnown<AssociationGroupInfoCCCommandListReport["commands"]>>;
    reportCommands(groupId: number, commands: ReadonlyMap<CommandClasses, readonly number[]>): Promise<void>;
}
export declare class AssociationGroupInfoCC extends CommandClass {
    ccCommand: AssociationGroupInfoCommand;
    determineRequiredCCInterviews(): readonly CommandClasses[];
    /** Returns the name of an association group */
    static getGroupNameCached(applHost: ZWaveApplicationHost, endpoint: IZWaveEndpoint, groupId: number): MaybeNotKnown<string>;
    /** Returns the association profile for an association group */
    static getGroupProfileCached(applHost: ZWaveApplicationHost, endpoint: IZWaveEndpoint, groupId: number): MaybeNotKnown<AssociationGroupInfoProfile>;
    /** Returns the dictionary of all commands issued by the given association group */
    static getIssuedCommandsCached(applHost: ZWaveApplicationHost, endpoint: IZWaveEndpoint, groupId: number): MaybeNotKnown<ReadonlyMap<CommandClasses, readonly number[]>>;
    static findGroupsForIssuedCommand(applHost: ZWaveApplicationHost, endpoint: IZWaveEndpoint, ccId: CommandClasses, command: number): number[];
    private static getAssociationGroupCountCached;
    interview(applHost: ZWaveApplicationHost): Promise<void>;
    refreshValues(applHost: ZWaveApplicationHost): Promise<void>;
}
export interface AssociationGroupInfoCCNameReportOptions extends CCCommandOptions {
    groupId: number;
    name: string;
}
export declare class AssociationGroupInfoCCNameReport extends AssociationGroupInfoCC {
    constructor(host: ZWaveHost, options: CommandClassDeserializationOptions | AssociationGroupInfoCCNameReportOptions);
    readonly groupId: number;
    readonly name: string;
    persistValues(applHost: ZWaveApplicationHost): boolean;
    serialize(): Buffer;
    toLogEntry(applHost: ZWaveApplicationHost): MessageOrCCLogEntry;
}
interface AssociationGroupInfoCCNameGetOptions extends CCCommandOptions {
    groupId: number;
}
export declare class AssociationGroupInfoCCNameGet extends AssociationGroupInfoCC {
    constructor(host: ZWaveHost, options: CommandClassDeserializationOptions | AssociationGroupInfoCCNameGetOptions);
    groupId: number;
    serialize(): Buffer;
    toLogEntry(applHost: ZWaveApplicationHost): MessageOrCCLogEntry;
}
export interface AssociationGroupInfo {
    groupId: number;
    mode: number;
    profile: number;
    eventCode: number;
}
export interface AssociationGroupInfoCCInfoReportSpecificOptions {
    isListMode: boolean;
    hasDynamicInfo: boolean;
    groups: AssociationGroupInfo[];
}
export declare class AssociationGroupInfoCCInfoReport extends AssociationGroupInfoCC {
    constructor(host: ZWaveHost, options: CommandClassDeserializationOptions | (AssociationGroupInfoCCInfoReportSpecificOptions & CCCommandOptions));
    readonly isListMode: boolean;
    readonly hasDynamicInfo: boolean;
    readonly groups: readonly AssociationGroupInfo[];
    persistValues(applHost: ZWaveApplicationHost): boolean;
    serialize(): Buffer;
    toLogEntry(applHost: ZWaveApplicationHost): MessageOrCCLogEntry;
}
type AssociationGroupInfoCCInfoGetOptions = CCCommandOptions & {
    refreshCache: boolean;
} & ({
    listMode: boolean;
} | {
    groupId: number;
});
export declare class AssociationGroupInfoCCInfoGet extends AssociationGroupInfoCC {
    constructor(host: ZWaveHost, options: CommandClassDeserializationOptions | AssociationGroupInfoCCInfoGetOptions);
    refreshCache: boolean;
    listMode?: boolean;
    groupId?: number;
    serialize(): Buffer;
    toLogEntry(applHost: ZWaveApplicationHost): MessageOrCCLogEntry;
}
export interface AssociationGroupInfoCCCommandListReportOptions extends CCCommandOptions {
    groupId: number;
    commands: ReadonlyMap<CommandClasses, readonly number[]>;
}
export declare class AssociationGroupInfoCCCommandListReport extends AssociationGroupInfoCC {
    constructor(host: ZWaveHost, options: CommandClassDeserializationOptions | AssociationGroupInfoCCCommandListReportOptions);
    readonly groupId: number;
    readonly commands: ReadonlyMap<CommandClasses, readonly number[]>;
    serialize(): Buffer;
    toLogEntry(applHost: ZWaveApplicationHost): MessageOrCCLogEntry;
}
interface AssociationGroupInfoCCCommandListGetOptions extends CCCommandOptions {
    allowCache: boolean;
    groupId: number;
}
export declare class AssociationGroupInfoCCCommandListGet extends AssociationGroupInfoCC {
    constructor(host: ZWaveHost, options: CommandClassDeserializationOptions | AssociationGroupInfoCCCommandListGetOptions);
    allowCache: boolean;
    groupId: number;
    serialize(): Buffer;
    toLogEntry(applHost: ZWaveApplicationHost): MessageOrCCLogEntry;
}
export {};
//# sourceMappingURL=AssociationGroupInfoCC.d.ts.map