/// <reference types="node" />
import { CommandClasses, type IZWaveEndpoint, type MessageOrCCLogEntry, type SupervisionResult } from "@zwave-js/core";
import { type MaybeNotKnown } from "@zwave-js/core/safe";
import type { ZWaveApplicationHost, ZWaveHost } from "@zwave-js/host";
import { type AllOrNone } from "@zwave-js/shared";
import { CCAPI } from "../lib/API";
import { type CCCommandOptions, CommandClass, type CommandClassDeserializationOptions } from "../lib/CommandClass";
import { ScheduleEntryLockCommand, type ScheduleEntryLockDailyRepeatingSchedule, ScheduleEntryLockScheduleKind, ScheduleEntryLockSetAction, type ScheduleEntryLockSlotId, type ScheduleEntryLockWeekDaySchedule, ScheduleEntryLockWeekday, type ScheduleEntryLockYearDaySchedule, type Timezone } from "../lib/_Types";
export declare const ScheduleEntryLockCCValues: Readonly<{
    schedule: ((scheduleKind: ScheduleEntryLockScheduleKind, userId: number, slotId: number) => {
        readonly meta: {
            readonly type: "any";
            readonly readable: true;
            readonly writeable: true;
        };
        readonly id: {
            commandClass: (typeof CommandClasses)["Schedule Entry Lock"];
            property: "schedule";
            propertyKey: number;
        };
        readonly endpoint: (endpoint?: number | undefined) => {
            readonly commandClass: (typeof CommandClasses)["Schedule Entry Lock"];
            readonly endpoint: number;
            readonly property: "schedule";
            readonly propertyKey: number;
        };
    }) & {
        is: (valueId: import("@zwave-js/core").ValueID) => boolean;
        readonly options: {
            readonly stateful: true;
            readonly secret: false;
            readonly minVersion: 1;
            readonly supportsEndpoints: true;
            readonly autoCreate: true;
            readonly internal: true;
        };
    };
    scheduleKind: ((userId: number) => {
        readonly meta: {
            readonly type: "any";
            readonly readable: true;
            readonly writeable: true;
        };
        readonly id: {
            commandClass: (typeof CommandClasses)["Schedule Entry Lock"];
            property: "scheduleKind";
            propertyKey: number;
        };
        readonly endpoint: (endpoint?: number | undefined) => {
            readonly commandClass: (typeof CommandClasses)["Schedule Entry Lock"];
            readonly endpoint: number;
            readonly property: "scheduleKind";
            readonly propertyKey: number;
        };
    }) & {
        is: (valueId: import("@zwave-js/core").ValueID) => boolean;
        readonly options: {
            readonly stateful: true;
            readonly secret: false;
            readonly minVersion: 1;
            readonly supportsEndpoints: true;
            readonly autoCreate: true;
            readonly internal: true;
        };
    };
    userEnabled: ((userId: number) => {
        readonly meta: {
            readonly type: "any";
            readonly readable: true;
            readonly writeable: true;
        };
        readonly id: {
            commandClass: (typeof CommandClasses)["Schedule Entry Lock"];
            property: "userEnabled";
            propertyKey: number;
        };
        readonly endpoint: (endpoint?: number | undefined) => {
            readonly commandClass: (typeof CommandClasses)["Schedule Entry Lock"];
            readonly endpoint: number;
            readonly property: "userEnabled";
            readonly propertyKey: number;
        };
    }) & {
        is: (valueId: import("@zwave-js/core").ValueID) => boolean;
        readonly options: {
            readonly stateful: true;
            readonly secret: false;
            readonly minVersion: 1;
            readonly supportsEndpoints: true;
            readonly autoCreate: true;
            readonly internal: true;
        };
    };
    numDailyRepeatingSlots: {
        readonly id: {
            commandClass: (typeof CommandClasses)["Schedule Entry Lock"];
            property: "numDailyRepeatingSlots";
        };
        readonly endpoint: (endpoint?: number | undefined) => {
            readonly commandClass: (typeof CommandClasses)["Schedule Entry Lock"];
            readonly endpoint: number;
            readonly property: "numDailyRepeatingSlots";
        };
        readonly is: (valueId: import("@zwave-js/core").ValueID) => boolean;
        readonly meta: {
            readonly type: "any";
            readonly readable: true;
            readonly writeable: true;
        };
        readonly options: {
            readonly stateful: true;
            readonly secret: false;
            readonly minVersion: 1;
            readonly supportsEndpoints: true;
            readonly autoCreate: true;
            readonly internal: true;
        };
    };
    numYearDaySlots: {
        readonly id: {
            commandClass: (typeof CommandClasses)["Schedule Entry Lock"];
            property: "numYearDaySlots";
        };
        readonly endpoint: (endpoint?: number | undefined) => {
            readonly commandClass: (typeof CommandClasses)["Schedule Entry Lock"];
            readonly endpoint: number;
            readonly property: "numYearDaySlots";
        };
        readonly is: (valueId: import("@zwave-js/core").ValueID) => boolean;
        readonly meta: {
            readonly type: "any";
            readonly readable: true;
            readonly writeable: true;
        };
        readonly options: {
            readonly stateful: true;
            readonly secret: false;
            readonly minVersion: 1;
            readonly supportsEndpoints: true;
            readonly autoCreate: true;
            readonly internal: true;
        };
    };
    numWeekDaySlots: {
        readonly id: {
            commandClass: (typeof CommandClasses)["Schedule Entry Lock"];
            property: "numWeekDaySlots";
        };
        readonly endpoint: (endpoint?: number | undefined) => {
            readonly commandClass: (typeof CommandClasses)["Schedule Entry Lock"];
            readonly endpoint: number;
            readonly property: "numWeekDaySlots";
        };
        readonly is: (valueId: import("@zwave-js/core").ValueID) => boolean;
        readonly meta: {
            readonly type: "any";
            readonly readable: true;
            readonly writeable: true;
        };
        readonly options: {
            readonly stateful: true;
            readonly secret: false;
            readonly minVersion: 1;
            readonly supportsEndpoints: true;
            readonly autoCreate: true;
            readonly internal: true;
        };
    };
}>;
export declare class ScheduleEntryLockCCAPI extends CCAPI {
    supportsCommand(cmd: ScheduleEntryLockCommand): MaybeNotKnown<boolean>;
    /**
     * Enables or disables schedules. If a user ID is given, that user's
     * schedules will be enabled or disabled. If no user ID is given, all schedules
     * will be affected.
     */
    setEnabled(enabled: boolean, userId?: number): Promise<SupervisionResult | undefined>;
    getNumSlots(): Promise<Pick<ScheduleEntryLockCCSupportedReport, "numWeekDaySlots" | "numYearDaySlots" | "numDailyRepeatingSlots"> | undefined>;
    setWeekDaySchedule(slot: ScheduleEntryLockSlotId, schedule?: ScheduleEntryLockWeekDaySchedule): Promise<SupervisionResult | undefined>;
    getWeekDaySchedule(slot: ScheduleEntryLockSlotId): Promise<MaybeNotKnown<ScheduleEntryLockWeekDaySchedule>>;
    setYearDaySchedule(slot: ScheduleEntryLockSlotId, schedule?: ScheduleEntryLockYearDaySchedule): Promise<SupervisionResult | undefined>;
    getYearDaySchedule(slot: ScheduleEntryLockSlotId): Promise<MaybeNotKnown<ScheduleEntryLockYearDaySchedule>>;
    setDailyRepeatingSchedule(slot: ScheduleEntryLockSlotId, schedule?: ScheduleEntryLockDailyRepeatingSchedule): Promise<SupervisionResult | undefined>;
    getDailyRepeatingSchedule(slot: ScheduleEntryLockSlotId): Promise<MaybeNotKnown<ScheduleEntryLockDailyRepeatingSchedule>>;
    getTimezone(): Promise<MaybeNotKnown<Timezone>>;
    setTimezone(timezone: Timezone): Promise<SupervisionResult | undefined>;
}
export declare class ScheduleEntryLockCC extends CommandClass {
    ccCommand: ScheduleEntryLockCommand;
    interview(applHost: ZWaveApplicationHost): Promise<void>;
    /**
     * Returns the number of supported day-of-week slots.
     * This only works AFTER the interview process
     */
    static getNumWeekDaySlotsCached(applHost: ZWaveApplicationHost, endpoint: IZWaveEndpoint): number;
    /**
     * Returns the number of supported day-of-year slots.
     * This only works AFTER the interview process
     */
    static getNumYearDaySlotsCached(applHost: ZWaveApplicationHost, endpoint: IZWaveEndpoint): number;
    /**
     * Returns the number of supported daily-repeating slots.
     * This only works AFTER the interview process
     */
    static getNumDailyRepeatingSlotsCached(applHost: ZWaveApplicationHost, endpoint: IZWaveEndpoint): number;
    /**
     * Returns whether scheduling for a given user ID (most likely) enabled. Since the Schedule Entry Lock CC
     * provides no way to query the enabled state, Z-Wave JS tracks this in its own cache.
     *
     * This only works AFTER the interview process and is likely to be wrong if a device
     * with existing schedules is queried. To be sure, disable scheduling for all users and enable
     * only the desired ones.
     */
    static getUserCodeScheduleEnabledCached(applHost: ZWaveApplicationHost, endpoint: IZWaveEndpoint, userId: number): boolean;
    /**
     * Returns which scheduling kind is (most likely) enabled for a given user ID . Since the Schedule Entry Lock CC
     * provides no way to query the current state, Z-Wave JS tracks this in its own cache.
     *
     * This only works AFTER the interview process and is likely to be wrong if a device
     * with existing schedules is queried. To be sure, edit a schedule of the desired kind
     * which will automatically switch the user to that scheduling kind.
     */
    static getUserCodeScheduleKindCached(applHost: ZWaveApplicationHost, endpoint: IZWaveEndpoint, userId: number): MaybeNotKnown<ScheduleEntryLockScheduleKind>;
    static getScheduleCached(applHost: ZWaveApplicationHost, endpoint: IZWaveEndpoint, scheduleKind: ScheduleEntryLockScheduleKind.WeekDay, userId: number, slotId: number): MaybeNotKnown<ScheduleEntryLockWeekDaySchedule | false>;
    static getScheduleCached(applHost: ZWaveApplicationHost, endpoint: IZWaveEndpoint, scheduleKind: ScheduleEntryLockScheduleKind.YearDay, userId: number, slotId: number): MaybeNotKnown<ScheduleEntryLockYearDaySchedule | false>;
    static getScheduleCached(applHost: ZWaveApplicationHost, endpoint: IZWaveEndpoint, scheduleKind: ScheduleEntryLockScheduleKind.DailyRepeating, userId: number, slotId: number): MaybeNotKnown<ScheduleEntryLockDailyRepeatingSchedule | false>;
    static getScheduleCached(applHost: ZWaveApplicationHost, endpoint: IZWaveEndpoint, scheduleKind: ScheduleEntryLockScheduleKind, userId: number, slotId: number): MaybeNotKnown<ScheduleEntryLockWeekDaySchedule | ScheduleEntryLockYearDaySchedule | ScheduleEntryLockDailyRepeatingSchedule | false>;
}
interface ScheduleEntryLockCCEnableSetOptions extends CCCommandOptions {
    userId: number;
    enabled: boolean;
}
export declare class ScheduleEntryLockCCEnableSet extends ScheduleEntryLockCC {
    constructor(host: ZWaveHost, options: CommandClassDeserializationOptions | ScheduleEntryLockCCEnableSetOptions);
    userId: number;
    enabled: boolean;
    serialize(): Buffer;
    toLogEntry(applHost: ZWaveApplicationHost): MessageOrCCLogEntry;
}
interface ScheduleEntryLockCCEnableAllSetOptions extends CCCommandOptions {
    enabled: boolean;
}
export declare class ScheduleEntryLockCCEnableAllSet extends ScheduleEntryLockCC {
    constructor(host: ZWaveHost, options: CommandClassDeserializationOptions | ScheduleEntryLockCCEnableAllSetOptions);
    enabled: boolean;
    serialize(): Buffer;
    toLogEntry(applHost: ZWaveApplicationHost): MessageOrCCLogEntry;
}
interface ScheduleEntryLockCCSupportedReportOptions extends CCCommandOptions {
    numWeekDaySlots: number;
    numYearDaySlots: number;
    numDailyRepeatingSlots?: number;
}
export declare class ScheduleEntryLockCCSupportedReport extends ScheduleEntryLockCC {
    constructor(host: ZWaveHost, options: CommandClassDeserializationOptions | ScheduleEntryLockCCSupportedReportOptions);
    numWeekDaySlots: number;
    numYearDaySlots: number;
    numDailyRepeatingSlots: number | undefined;
    serialize(): Buffer;
    toLogEntry(applHost: ZWaveApplicationHost): MessageOrCCLogEntry;
}
export declare class ScheduleEntryLockCCSupportedGet extends ScheduleEntryLockCC {
}
/** @publicAPI */
export type ScheduleEntryLockCCWeekDayScheduleSetOptions = CCCommandOptions & ScheduleEntryLockSlotId & ({
    action: ScheduleEntryLockSetAction.Erase;
} | ({
    action: ScheduleEntryLockSetAction.Set;
} & ScheduleEntryLockWeekDaySchedule));
export declare class ScheduleEntryLockCCWeekDayScheduleSet extends ScheduleEntryLockCC {
    constructor(host: ZWaveHost, options: CommandClassDeserializationOptions | ScheduleEntryLockCCWeekDayScheduleSetOptions);
    userId: number;
    slotId: number;
    action: ScheduleEntryLockSetAction;
    weekday?: ScheduleEntryLockWeekday;
    startHour?: number;
    startMinute?: number;
    stopHour?: number;
    stopMinute?: number;
    serialize(): Buffer;
    toLogEntry(applHost: ZWaveApplicationHost): MessageOrCCLogEntry;
}
type ScheduleEntryLockCCWeekDayScheduleReportOptions = CCCommandOptions & ScheduleEntryLockSlotId & AllOrNone<ScheduleEntryLockWeekDaySchedule>;
export declare class ScheduleEntryLockCCWeekDayScheduleReport extends ScheduleEntryLockCC {
    constructor(host: ZWaveHost, options: CommandClassDeserializationOptions | ScheduleEntryLockCCWeekDayScheduleReportOptions);
    userId: number;
    slotId: number;
    weekday?: ScheduleEntryLockWeekday;
    startHour?: number;
    startMinute?: number;
    stopHour?: number;
    stopMinute?: number;
    persistValues(applHost: ZWaveApplicationHost): boolean;
    serialize(): Buffer;
    toLogEntry(applHost: ZWaveApplicationHost): MessageOrCCLogEntry;
}
type ScheduleEntryLockCCWeekDayScheduleGetOptions = CCCommandOptions & ScheduleEntryLockSlotId;
export declare class ScheduleEntryLockCCWeekDayScheduleGet extends ScheduleEntryLockCC {
    constructor(host: ZWaveHost, options: CommandClassDeserializationOptions | ScheduleEntryLockCCWeekDayScheduleGetOptions);
    userId: number;
    slotId: number;
    serialize(): Buffer;
    toLogEntry(applHost: ZWaveApplicationHost): MessageOrCCLogEntry;
}
/** @publicAPI */
export type ScheduleEntryLockCCYearDayScheduleSetOptions = CCCommandOptions & ScheduleEntryLockSlotId & ({
    action: ScheduleEntryLockSetAction.Erase;
} | ({
    action: ScheduleEntryLockSetAction.Set;
} & ScheduleEntryLockYearDaySchedule));
export declare class ScheduleEntryLockCCYearDayScheduleSet extends ScheduleEntryLockCC {
    constructor(host: ZWaveHost, options: CommandClassDeserializationOptions | ScheduleEntryLockCCYearDayScheduleSetOptions);
    userId: number;
    slotId: number;
    action: ScheduleEntryLockSetAction;
    startYear?: number;
    startMonth?: number;
    startDay?: number;
    startHour?: number;
    startMinute?: number;
    stopYear?: number;
    stopMonth?: number;
    stopDay?: number;
    stopHour?: number;
    stopMinute?: number;
    serialize(): Buffer;
    toLogEntry(applHost: ZWaveApplicationHost): MessageOrCCLogEntry;
}
type ScheduleEntryLockCCYearDayScheduleReportOptions = CCCommandOptions & ScheduleEntryLockSlotId & AllOrNone<ScheduleEntryLockYearDaySchedule>;
export declare class ScheduleEntryLockCCYearDayScheduleReport extends ScheduleEntryLockCC {
    constructor(host: ZWaveHost, options: CommandClassDeserializationOptions | ScheduleEntryLockCCYearDayScheduleReportOptions);
    userId: number;
    slotId: number;
    startYear?: number;
    startMonth?: number;
    startDay?: number;
    startHour?: number;
    startMinute?: number;
    stopYear?: number;
    stopMonth?: number;
    stopDay?: number;
    stopHour?: number;
    stopMinute?: number;
    persistValues(applHost: ZWaveApplicationHost): boolean;
    serialize(): Buffer;
    toLogEntry(applHost: ZWaveApplicationHost): MessageOrCCLogEntry;
}
type ScheduleEntryLockCCYearDayScheduleGetOptions = CCCommandOptions & ScheduleEntryLockSlotId;
export declare class ScheduleEntryLockCCYearDayScheduleGet extends ScheduleEntryLockCC {
    constructor(host: ZWaveHost, options: CommandClassDeserializationOptions | ScheduleEntryLockCCYearDayScheduleGetOptions);
    userId: number;
    slotId: number;
    serialize(): Buffer;
    toLogEntry(applHost: ZWaveApplicationHost): MessageOrCCLogEntry;
}
interface ScheduleEntryLockCCTimeOffsetSetOptions extends CCCommandOptions {
    standardOffset: number;
    dstOffset: number;
}
export declare class ScheduleEntryLockCCTimeOffsetSet extends ScheduleEntryLockCC {
    constructor(host: ZWaveHost, options: CommandClassDeserializationOptions | ScheduleEntryLockCCTimeOffsetSetOptions);
    standardOffset: number;
    dstOffset: number;
    serialize(): Buffer;
    toLogEntry(applHost: ZWaveApplicationHost): MessageOrCCLogEntry;
}
interface ScheduleEntryLockCCTimeOffsetReportOptions extends CCCommandOptions {
    standardOffset: number;
    dstOffset: number;
}
export declare class ScheduleEntryLockCCTimeOffsetReport extends ScheduleEntryLockCC {
    constructor(host: ZWaveHost, options: CommandClassDeserializationOptions | ScheduleEntryLockCCTimeOffsetReportOptions);
    standardOffset: number;
    dstOffset: number;
    serialize(): Buffer;
    toLogEntry(applHost: ZWaveApplicationHost): MessageOrCCLogEntry;
}
export declare class ScheduleEntryLockCCTimeOffsetGet extends ScheduleEntryLockCC {
}
/** @publicAPI */
export type ScheduleEntryLockCCDailyRepeatingScheduleSetOptions = CCCommandOptions & ScheduleEntryLockSlotId & ({
    action: ScheduleEntryLockSetAction.Erase;
} | ({
    action: ScheduleEntryLockSetAction.Set;
} & ScheduleEntryLockDailyRepeatingSchedule));
export declare class ScheduleEntryLockCCDailyRepeatingScheduleSet extends ScheduleEntryLockCC {
    constructor(host: ZWaveHost, options: CommandClassDeserializationOptions | ScheduleEntryLockCCDailyRepeatingScheduleSetOptions);
    userId: number;
    slotId: number;
    action: ScheduleEntryLockSetAction;
    weekdays?: ScheduleEntryLockWeekday[];
    startHour?: number;
    startMinute?: number;
    durationHour?: number;
    durationMinute?: number;
    serialize(): Buffer;
    toLogEntry(applHost: ZWaveApplicationHost): MessageOrCCLogEntry;
}
type ScheduleEntryLockCCDailyRepeatingScheduleReportOptions = ScheduleEntryLockSlotId & AllOrNone<ScheduleEntryLockDailyRepeatingSchedule>;
export declare class ScheduleEntryLockCCDailyRepeatingScheduleReport extends ScheduleEntryLockCC {
    constructor(host: ZWaveHost, options: CommandClassDeserializationOptions | (CCCommandOptions & ScheduleEntryLockCCDailyRepeatingScheduleReportOptions));
    userId: number;
    slotId: number;
    weekdays?: ScheduleEntryLockWeekday[];
    startHour?: number;
    startMinute?: number;
    durationHour?: number;
    durationMinute?: number;
    persistValues(applHost: ZWaveApplicationHost): boolean;
    serialize(): Buffer;
    toLogEntry(applHost: ZWaveApplicationHost): MessageOrCCLogEntry;
}
type ScheduleEntryLockCCDailyRepeatingScheduleGetOptions = CCCommandOptions & ScheduleEntryLockSlotId;
export declare class ScheduleEntryLockCCDailyRepeatingScheduleGet extends ScheduleEntryLockCC {
    constructor(host: ZWaveHost, options: CommandClassDeserializationOptions | ScheduleEntryLockCCDailyRepeatingScheduleGetOptions);
    userId: number;
    slotId: number;
    serialize(): Buffer;
    toLogEntry(applHost: ZWaveApplicationHost): MessageOrCCLogEntry;
}
export {};
//# sourceMappingURL=ScheduleEntryLockCC.d.ts.map