/// <reference types="node" />
import { CommandClasses, type MaybeNotKnown, type MessageOrCCLogEntry, type SupervisionResult } from "@zwave-js/core/safe";
import type { ZWaveApplicationHost, ZWaveHost } from "@zwave-js/host/safe";
import { CCAPI, POLL_VALUE, type PollValueImplementation, SET_VALUE, type SetValueImplementation } from "../lib/API";
import { type CCCommandOptions, CommandClass, type CommandClassDeserializationOptions } from "../lib/CommandClass";
import { ThermostatMode, ThermostatModeCommand } from "../lib/_Types";
export declare const ThermostatModeCCValues: Readonly<{
    supportedModes: {
        readonly id: {
            commandClass: (typeof CommandClasses)["Thermostat Mode"];
            property: "supportedModes";
        };
        readonly endpoint: (endpoint?: number | undefined) => {
            readonly commandClass: (typeof CommandClasses)["Thermostat Mode"];
            readonly endpoint: number;
            readonly property: "supportedModes";
        };
        readonly is: (valueId: import("@zwave-js/core/safe").ValueID) => boolean;
        readonly meta: {
            readonly type: "any";
            readonly readable: true;
            readonly writeable: true;
        };
        readonly options: {
            readonly stateful: true;
            readonly secret: false;
            readonly minVersion: 1;
            readonly supportsEndpoints: true;
            readonly autoCreate: true;
            readonly internal: true;
        };
    };
    manufacturerData: {
        readonly id: {
            commandClass: (typeof CommandClasses)["Thermostat Mode"];
            property: "manufacturerData";
        };
        readonly endpoint: (endpoint?: number | undefined) => {
            readonly commandClass: (typeof CommandClasses)["Thermostat Mode"];
            readonly endpoint: number;
            readonly property: "manufacturerData";
        };
        readonly is: (valueId: import("@zwave-js/core/safe").ValueID) => boolean;
        readonly meta: {
            readonly label: "Manufacturer data";
            readonly writeable: false;
            readonly type: "buffer";
            readonly readable: true;
        };
        readonly options: {
            readonly internal: false;
            readonly minVersion: 1;
            readonly secret: false;
            readonly stateful: true;
            readonly supportsEndpoints: true;
            readonly autoCreate: true;
        };
    };
    thermostatMode: {
        readonly id: {
            commandClass: (typeof CommandClasses)["Thermostat Mode"];
            property: "mode";
        };
        readonly endpoint: (endpoint?: number | undefined) => {
            readonly commandClass: (typeof CommandClasses)["Thermostat Mode"];
            readonly endpoint: number;
            readonly property: "mode";
        };
        readonly is: (valueId: import("@zwave-js/core/safe").ValueID) => boolean;
        readonly meta: {
            readonly states: {
                [x: number]: string;
            };
            readonly label: "Thermostat mode";
            readonly min: 0;
            readonly max: 255;
            readonly type: "number";
            readonly readable: true;
            readonly writeable: true;
        };
        readonly options: {
            readonly internal: false;
            readonly minVersion: 1;
            readonly secret: false;
            readonly stateful: true;
            readonly supportsEndpoints: true;
            readonly autoCreate: true;
        };
    };
}>;
export declare class ThermostatModeCCAPI extends CCAPI {
    supportsCommand(cmd: ThermostatModeCommand): MaybeNotKnown<boolean>;
    protected get [SET_VALUE](): SetValueImplementation;
    protected get [POLL_VALUE](): PollValueImplementation;
    get(): Promise<Pick<ThermostatModeCCReport, "mode" | "manufacturerData"> | undefined>;
    set(mode: Exclude<ThermostatMode, (typeof ThermostatMode)["Manufacturer specific"]>): Promise<SupervisionResult | undefined>;
    set(mode: (typeof ThermostatMode)["Manufacturer specific"], manufacturerData: Buffer | string): Promise<SupervisionResult | undefined>;
    getSupportedModes(): Promise<MaybeNotKnown<readonly ThermostatMode[]>>;
}
export declare class ThermostatModeCC extends CommandClass {
    ccCommand: ThermostatModeCommand;
    interview(applHost: ZWaveApplicationHost): Promise<void>;
    refreshValues(applHost: ZWaveApplicationHost): Promise<void>;
}
export type ThermostatModeCCSetOptions = CCCommandOptions & ({
    mode: Exclude<ThermostatMode, (typeof ThermostatMode)["Manufacturer specific"]>;
} | {
    mode: (typeof ThermostatMode)["Manufacturer specific"];
    manufacturerData: Buffer;
});
export declare class ThermostatModeCCSet extends ThermostatModeCC {
    constructor(host: ZWaveHost, options: CommandClassDeserializationOptions | ThermostatModeCCSetOptions);
    mode: ThermostatMode;
    manufacturerData?: Buffer;
    serialize(): Buffer;
    toLogEntry(applHost: ZWaveApplicationHost): MessageOrCCLogEntry;
}
export type ThermostatModeCCReportOptions = CCCommandOptions & ({
    mode: Exclude<ThermostatMode, (typeof ThermostatMode)["Manufacturer specific"]>;
    manufacturerData?: undefined;
} | {
    mode: (typeof ThermostatMode)["Manufacturer specific"];
    manufacturerData?: Buffer;
});
export declare class ThermostatModeCCReport extends ThermostatModeCC {
    constructor(host: ZWaveHost, options: CommandClassDeserializationOptions | ThermostatModeCCReportOptions);
    persistValues(applHost: ZWaveApplicationHost): boolean;
    readonly mode: ThermostatMode;
    readonly manufacturerData: Buffer | undefined;
    serialize(): Buffer;
    toLogEntry(applHost: ZWaveApplicationHost): MessageOrCCLogEntry;
}
export declare class ThermostatModeCCGet extends ThermostatModeCC {
}
export interface ThermostatModeCCSupportedReportOptions extends CCCommandOptions {
    supportedModes: ThermostatMode[];
}
export declare class ThermostatModeCCSupportedReport extends ThermostatModeCC {
    constructor(host: ZWaveHost, options: CommandClassDeserializationOptions | ThermostatModeCCSupportedReportOptions);
    persistValues(applHost: ZWaveApplicationHost): boolean;
    readonly supportedModes: ThermostatMode[];
    serialize(): Buffer;
    toLogEntry(applHost: ZWaveApplicationHost): MessageOrCCLogEntry;
}
export declare class ThermostatModeCCSupportedGet extends ThermostatModeCC {
}
//# sourceMappingURL=ThermostatModeCC.d.ts.map