/// <reference types="node" />
import { CommandClasses, Duration, type MessageOrCCLogEntry, type SupervisionResult } from "@zwave-js/core";
import { type MaybeNotKnown } from "@zwave-js/core/safe";
import type { ZWaveApplicationHost, ZWaveHost } from "@zwave-js/host";
import { CCAPI, POLL_VALUE, type PollValueImplementation, SET_VALUE, SET_VALUE_HOOKS, type SetValueImplementation, type SetValueImplementationHooksFactory } from "../lib/API";
import { type CCCommandOptions, CommandClass, type CommandClassDeserializationOptions } from "../lib/CommandClass";
import { type LevelChangeDirection, WindowCoveringCommand, WindowCoveringParameter } from "../lib/_Types";
export declare const WindowCoveringCCValues: Readonly<{
    levelChangeDown: ((parameter: WindowCoveringParameter) => {
        readonly meta: {
            readonly label: `${string} - ${string}`;
            readonly valueChangeOptions: readonly ["transitionDuration"];
            readonly states: {
                readonly true: "Start";
                readonly false: "Stop";
            };
            readonly ccSpecific: {
                readonly parameter: WindowCoveringParameter;
            };
            readonly readable: false;
            readonly type: "boolean";
            readonly writeable: true;
        };
        readonly id: {
            commandClass: (typeof CommandClasses)["Window Covering"];
            property: "levelChangeDown";
            propertyKey: WindowCoveringParameter;
        };
        readonly endpoint: (endpoint?: number | undefined) => {
            readonly commandClass: (typeof CommandClasses)["Window Covering"];
            readonly endpoint: number;
            readonly property: "levelChangeDown";
            readonly propertyKey: WindowCoveringParameter;
        };
    }) & {
        is: (valueId: import("@zwave-js/core").ValueID) => boolean;
        readonly options: {
            readonly internal: false;
            readonly minVersion: 1;
            readonly secret: false;
            readonly stateful: true;
            readonly supportsEndpoints: true;
            readonly autoCreate: true;
        };
    };
    levelChangeUp: ((parameter: WindowCoveringParameter) => {
        readonly meta: {
            readonly label: `${string} - ${string}`;
            readonly valueChangeOptions: readonly ["transitionDuration"];
            readonly states: {
                readonly true: "Start";
                readonly false: "Stop";
            };
            readonly ccSpecific: {
                readonly parameter: WindowCoveringParameter;
            };
            readonly readable: false;
            readonly type: "boolean";
            readonly writeable: true;
        };
        readonly id: {
            commandClass: (typeof CommandClasses)["Window Covering"];
            property: "levelChangeUp";
            propertyKey: WindowCoveringParameter;
        };
        readonly endpoint: (endpoint?: number | undefined) => {
            readonly commandClass: (typeof CommandClasses)["Window Covering"];
            readonly endpoint: number;
            readonly property: "levelChangeUp";
            readonly propertyKey: WindowCoveringParameter;
        };
    }) & {
        is: (valueId: import("@zwave-js/core").ValueID) => boolean;
        readonly options: {
            readonly internal: false;
            readonly minVersion: 1;
            readonly secret: false;
            readonly stateful: true;
            readonly supportsEndpoints: true;
            readonly autoCreate: true;
        };
    };
    duration: ((parameter: WindowCoveringParameter) => {
        readonly meta: {
            readonly label: `Remaining duration - ${string}`;
            readonly ccSpecific: {
                readonly parameter: WindowCoveringParameter;
            };
            readonly writeable: false;
            readonly type: "duration";
            readonly readable: true;
        };
        readonly id: {
            commandClass: (typeof CommandClasses)["Window Covering"];
            property: "duration";
            propertyKey: WindowCoveringParameter;
        };
        readonly endpoint: (endpoint?: number | undefined) => {
            readonly commandClass: (typeof CommandClasses)["Window Covering"];
            readonly endpoint: number;
            readonly property: "duration";
            readonly propertyKey: WindowCoveringParameter;
        };
    }) & {
        is: (valueId: import("@zwave-js/core").ValueID) => boolean;
        readonly options: {
            readonly internal: false;
            readonly minVersion: 1;
            readonly secret: false;
            readonly stateful: true;
            readonly supportsEndpoints: true;
            readonly autoCreate: true;
        };
    };
    targetValue: ((parameter: WindowCoveringParameter) => {
        readonly meta: {
            readonly label: `Target value - ${string}`;
            readonly writeable: boolean;
            readonly states: {
                [x: number]: string;
            };
            readonly allowManualEntry: boolean;
            readonly ccSpecific: {
                readonly parameter: WindowCoveringParameter;
            };
            readonly valueChangeOptions: readonly ["transitionDuration"];
            readonly max: 99;
            readonly min: 0;
            readonly type: "number";
            readonly readable: true;
        };
        readonly id: {
            commandClass: (typeof CommandClasses)["Window Covering"];
            property: "targetValue";
            propertyKey: WindowCoveringParameter;
        };
        readonly endpoint: (endpoint?: number | undefined) => {
            readonly commandClass: (typeof CommandClasses)["Window Covering"];
            readonly endpoint: number;
            readonly property: "targetValue";
            readonly propertyKey: WindowCoveringParameter;
        };
    }) & {
        is: (valueId: import("@zwave-js/core").ValueID) => boolean;
        readonly options: {
            readonly internal: false;
            readonly minVersion: 1;
            readonly secret: false;
            readonly stateful: true;
            readonly supportsEndpoints: true;
            readonly autoCreate: true;
        };
    };
    currentValue: ((parameter: WindowCoveringParameter) => {
        readonly meta: {
            readonly label: `Current value - ${string}`;
            readonly states: {
                [x: number]: string;
            };
            readonly ccSpecific: {
                readonly parameter: WindowCoveringParameter;
            };
            readonly writeable: false;
            readonly max: 99;
            readonly min: 0;
            readonly type: "number";
            readonly readable: true;
        };
        readonly id: {
            commandClass: (typeof CommandClasses)["Window Covering"];
            property: "currentValue";
            propertyKey: WindowCoveringParameter;
        };
        readonly endpoint: (endpoint?: number | undefined) => {
            readonly commandClass: (typeof CommandClasses)["Window Covering"];
            readonly endpoint: number;
            readonly property: "currentValue";
            readonly propertyKey: WindowCoveringParameter;
        };
    }) & {
        is: (valueId: import("@zwave-js/core").ValueID) => boolean;
        readonly options: {
            readonly internal: false;
            readonly minVersion: 1;
            readonly secret: false;
            readonly stateful: true;
            readonly supportsEndpoints: true;
            readonly autoCreate: true;
        };
    };
    supportedParameters: {
        readonly id: {
            commandClass: (typeof CommandClasses)["Window Covering"];
            property: "supportedParameters";
        };
        readonly endpoint: (endpoint?: number | undefined) => {
            readonly commandClass: (typeof CommandClasses)["Window Covering"];
            readonly endpoint: number;
            readonly property: "supportedParameters";
        };
        readonly is: (valueId: import("@zwave-js/core").ValueID) => boolean;
        readonly meta: {
            readonly type: "any";
            readonly readable: true;
            readonly writeable: true;
        };
        readonly options: {
            readonly stateful: true;
            readonly secret: false;
            readonly minVersion: 1;
            readonly supportsEndpoints: true;
            readonly autoCreate: true;
            readonly internal: true;
        };
    };
}>;
export declare class WindowCoveringCCAPI extends CCAPI {
    supportsCommand(cmd: WindowCoveringCommand): MaybeNotKnown<boolean>;
    protected get [SET_VALUE](): SetValueImplementation;
    protected [SET_VALUE_HOOKS]: SetValueImplementationHooksFactory;
    protected get [POLL_VALUE](): PollValueImplementation;
    getSupported(): Promise<MaybeNotKnown<readonly WindowCoveringParameter[]>>;
    get(parameter: WindowCoveringParameter): Promise<Pick<WindowCoveringCCReport, "duration" | "currentValue" | "targetValue"> | undefined>;
    set(targetValues: {
        parameter: WindowCoveringParameter;
        value: number;
    }[], duration?: Duration | string): Promise<SupervisionResult | undefined>;
    startLevelChange(parameter: WindowCoveringParameter, direction: keyof typeof LevelChangeDirection, duration?: Duration | string): Promise<SupervisionResult | undefined>;
    stopLevelChange(parameter: WindowCoveringParameter): Promise<SupervisionResult | undefined>;
}
export declare class WindowCoveringCC extends CommandClass {
    ccCommand: WindowCoveringCommand;
    interview(applHost: ZWaveApplicationHost): Promise<void>;
    translatePropertyKey(_applHost: ZWaveApplicationHost, _property: string | number, propertyKey: string | number): string | undefined;
}
export interface WindowCoveringCCSupportedReportOptions extends CCCommandOptions {
    supportedParameters: readonly WindowCoveringParameter[];
}
export declare class WindowCoveringCCSupportedReport extends WindowCoveringCC {
    constructor(host: ZWaveHost, options: CommandClassDeserializationOptions | WindowCoveringCCSupportedReportOptions);
    readonly supportedParameters: readonly WindowCoveringParameter[];
    serialize(): Buffer;
    toLogEntry(applHost: ZWaveApplicationHost): MessageOrCCLogEntry;
}
export declare class WindowCoveringCCSupportedGet extends WindowCoveringCC {
}
export declare class WindowCoveringCCReport extends WindowCoveringCC {
    constructor(host: ZWaveHost, options: CommandClassDeserializationOptions);
    readonly parameter: WindowCoveringParameter;
    readonly currentValue: number;
    readonly targetValue: number;
    readonly duration: Duration;
    toLogEntry(applHost: ZWaveApplicationHost): MessageOrCCLogEntry;
}
interface WindowCoveringCCGetOptions extends CCCommandOptions {
    parameter: WindowCoveringParameter;
}
export declare class WindowCoveringCCGet extends WindowCoveringCC {
    constructor(host: ZWaveHost, options: CommandClassDeserializationOptions | WindowCoveringCCGetOptions);
    parameter: WindowCoveringParameter;
    serialize(): Buffer;
    toLogEntry(applHost: ZWaveApplicationHost): MessageOrCCLogEntry;
}
export interface WindowCoveringCCSetOptions extends CCCommandOptions {
    targetValues: {
        parameter: WindowCoveringParameter;
        value: number;
    }[];
    duration?: Duration | string;
}
export declare class WindowCoveringCCSet extends WindowCoveringCC {
    constructor(host: ZWaveHost, options: CommandClassDeserializationOptions | WindowCoveringCCSetOptions);
    targetValues: {
        parameter: WindowCoveringParameter;
        value: number;
    }[];
    duration: Duration | undefined;
    serialize(): Buffer;
    toLogEntry(applHost: ZWaveApplicationHost): MessageOrCCLogEntry;
}
export interface WindowCoveringCCStartLevelChangeOptions extends CCCommandOptions {
    parameter: WindowCoveringParameter;
    direction: keyof typeof LevelChangeDirection;
    duration?: Duration | string;
}
export declare class WindowCoveringCCStartLevelChange extends WindowCoveringCC {
    constructor(host: ZWaveHost, options: CommandClassDeserializationOptions | WindowCoveringCCStartLevelChangeOptions);
    parameter: WindowCoveringParameter;
    direction: keyof typeof LevelChangeDirection;
    duration: Duration | undefined;
    serialize(): Buffer;
    toLogEntry(applHost: ZWaveApplicationHost): MessageOrCCLogEntry;
}
interface WindowCoveringCCStopLevelChangeOptions extends CCCommandOptions {
    parameter: WindowCoveringParameter;
}
export declare class WindowCoveringCCStopLevelChange extends WindowCoveringCC {
    constructor(host: ZWaveHost, options: CommandClassDeserializationOptions | WindowCoveringCCStopLevelChangeOptions);
    parameter: WindowCoveringParameter;
    serialize(): Buffer;
    toLogEntry(applHost: ZWaveApplicationHost): MessageOrCCLogEntry;
}
export {};
//# sourceMappingURL=WindowCoveringCC.d.ts.map